@@ -38,6 +38,10 @@
 
 
 extern int TIMEOUT;
+extern int KEEPALIVE_TIMEOUT;
+extern const char *DEFAULT_SERVICE;
+extern int PIPELINING;
+extern int CHECK_FOR_BUGGY_CLIENT;
 
 /*This variable defined in mpm_server.c and become 1 when the child must 
   halt imediatelly:*/
@@ -136,6 +140,23 @@ int recycle_request(ci_request_t * req, ci_connection_t * connection)
      return 1;
 }
 
+int keepalive_request(ci_request_t *req)
+{
+    /* Preserve extra read bytes*/
+    char *pstrblock = req->pstrblock_read;
+    int pstrblock_len = req->pstrblock_read_len;
+    // Just reset without change or free memory
+    ci_request_reset(req);
+    if (PIPELINING) {
+        req->pstrblock_read = pstrblock;
+        req->pstrblock_read_len = pstrblock_len;
+    }
+
+    if (req->pstrblock_read && req->pstrblock_read_len > 0)
+        return 1;
+    return ci_wait_for_data(req->connection->fd, KEEPALIVE_TIMEOUT, wait_for_read);
+}
+
 /*Here we want to read in small blocks icap header becouse in most cases
  it will not bigger than 512-1024 bytes.
  So we are going to do small reads and small increments in icap headers size,
@@ -164,20 +185,44 @@ static int icap_header_check_realloc(char **buf, int *size, int used, int mustad
 static int ci_read_icap_header(ci_request_t * req, ci_headers_list_t * h, int timeout)
 {
      int bytes, request_status = EC_100, i, eoh = 0, startsearch = 0, readed = 0;
-     int wait_status;
+     int wait_status = 0;
      char *buf_end;
+     int dataPrefetch = 0;
 
      buf_end = h->buf;
      readed = 0;
      bytes = 0;
 
-     while ((wait_status = wait_for_data(req->connection->fd, timeout, wait_for_read)) != CI_ERROR) {
-         bytes = ci_read_nonblock(req->connection->fd, buf_end, ICAP_HEADER_READSIZE);
-         if (bytes <= 0)
-             return EC_408;
+     if (PIPELINING && req->pstrblock_read && req->pstrblock_read_len > 0) {
+         if ((request_status =
+              icap_header_check_realloc(&(h->buf), &(h->bufsize), req->pstrblock_read_len,
+                                        ICAP_HEADER_READSIZE)) != EC_100)
+             return request_status;
+         memmove(h->buf, req->pstrblock_read, req->pstrblock_read_len);
+         readed = req->pstrblock_read_len;
+         buf_end = h->buf;
+         bytes = readed;
+         dataPrefetch = 1;
+         req->pstrblock_read = NULL;
+         req->pstrblock_read_len = 0;
+         ci_debug_printf(5, "Get data from previous request read.\n");
+     }
+
+     do {
+
+         if (!dataPrefetch) {
+             if ((wait_status = wait_for_data(req->connection->fd, timeout, wait_for_read)) == CI_ERROR)
+                 return EC_408;
+
+             bytes = ci_read_nonblock(req->connection->fd, buf_end, ICAP_HEADER_READSIZE);
+             if (bytes <= 0)
+                 return EC_408;
+
+             readed += bytes;
+             req->bytes_in += bytes;
+         } else
+             dataPrefetch = 0;
 
-          readed += bytes;
-          req->bytes_in += bytes;
           for (i = startsearch; i < bytes - 3; i++) {   /*search for end of header.... */
                if (strncmp(buf_end + i, "\r\n\r\n", 4) == 0) {
                     buf_end = buf_end + i + 2;
@@ -195,22 +240,21 @@ static int ci_read_icap_header(ci_request_t * req, ci_headers_list_t * h, int ti
           buf_end = h->buf + readed;
           if (startsearch > -3)
                startsearch = (readed > 3 ? -3 : -readed);       /*Including the last 3 char ellements ....... */
-     }
-     if (wait_status == CI_ERROR)
-          return EC_408;
+     } while(1);
+
      h->bufused = buf_end - h->buf;     /* -1 ; */
      req->pstrblock_read = buf_end + 2; /*after the \r\n\r\n. We keep the first \r\n and the other dropped.... */
      req->pstrblock_read_len = readed - h->bufused - 2; /*the 2 of the 4 characters \r\n\r\n and the '\0' character */
+     req->request_bytes_in = h->bufused + 2; /*This is include the "\r\n\r\n" sequence*/
      return request_status;
 }
 
-
 static int read_encaps_header(ci_request_t * req, ci_headers_list_t * h, int size)
 {
      int bytes = 0, remains, readed = 0;
      char *buf_end = NULL;
 
-     if (!ci_headers_setsize(h, size))
+     if (!ci_headers_setsize(h, size + (CHECK_FOR_BUGGY_CLIENT != 0 ? 2 : 0)))
           return EC_500;
      buf_end = h->buf;
 
@@ -243,9 +287,16 @@ static int read_encaps_header(ci_request_t * req, ci_headers_list_t * h, int siz
      h->bufused = buf_end - h->buf;     // -1 ;
      if (strncmp(buf_end - 4, "\r\n\r\n", 4) == 0) {
           h->bufused -= 2;      /*eat the last 2 bytes of "\r\n\r\n" */
+     } else if(CHECK_FOR_BUGGY_CLIENT && strncmp(buf_end - 2, "\r\n", 2) != 0) {
+         // Some icap clients missing the "\r\n\r\n" after end of headers
+         // when null-body is present.
+         *buf_end = '\r';
+         *(buf_end + 1) = '\n';
+         h->bufused += 2;
      }
      /*Currently we are counting only successfull http headers read.*/
      req->http_bytes_in += size;
+     req->request_bytes_in += size;
      return EC_100;
 }
 
@@ -305,15 +356,7 @@ static int parse_request(ci_request_t * req, char *buf)
          if (len) {
              strncpy(req->service, start, len);
              req->service[len] = '\0';
-             if (!(service = find_service(req->service))) { /*else search for an alias */
-                 if ((salias = find_service_alias(req->service))) {
-                     service = salias->service;
-                     if (salias->args[0] != '\0')
-                         strcpy(req->args, salias->args);
-                 }
-             }
          }
-         req->current_service_mod = service;
 
          if (*end == '?') {     /*args */
              start = ++end;
@@ -351,6 +394,21 @@ static int parse_request(ci_request_t * req, char *buf)
      if (vminor == -1 || vmajor < 1)
          return EC_400;
 
+     if (req->service[0] == '\0' && DEFAULT_SERVICE) { /*No service name defined*/
+         strncpy(req->service, DEFAULT_SERVICE, MAX_SERVICE_NAME);
+     }
+
+     if (req->service[0] != '\0') {
+         if (!(service = find_service(req->service))) { /*else search for an alias */
+             if ((salias = find_service_alias(req->service))) {
+                 service = salias->service;
+                 if (salias->args[0] != '\0')
+                     strcpy(req->args, salias->args);
+             }
+         }
+     }
+     req->current_service_mod = service;
+
      if (!req->current_service_mod)
          return EC_404; /*Service not found*/
 
@@ -418,6 +476,7 @@ static int parse_header(ci_request_t * req)
 {
      int i, request_status = EC_100, result;
      ci_headers_list_t *h;
+     char *val;
 
      h = req->request_header;
      if ((request_status = ci_read_icap_header(req, h, TIMEOUT)) != EC_100)
@@ -428,21 +487,26 @@ static int parse_header(ci_request_t * req)
 
      if ((request_status = parse_request(req, h->headers[0])) != EC_100)
          return request_status;
-	 
+
      for (i = 1; i < h->used && request_status == EC_100; i++) {
           if (strncasecmp("Preview:", h->headers[i], 8) == 0) {
-               result = strtol(h->headers[i] + 9, NULL, 10);
+               val = h->headers[i] + 8;
+               for (;isspace(*val) && *val != '\0'; ++val);
+               errno = 0;
+               result = strtol(val, NULL, 10);
                if (errno != EINVAL && errno != ERANGE) {
                     req->preview = result;
                     if (result >= 0)
                          ci_buf_reset_size(&(req->preview_data), result + 64);
                }
           }
-          else if (strncasecmp("Encapsulated: ", h->headers[i], 14) == 0)
+          else if (strncasecmp("Encapsulated:", h->headers[i], 13) == 0)
                request_status = process_encapsulated(req, h->headers[i]);
-          else if (strncasecmp("Connection: ", h->headers[i], 12) == 0) {
-/*	       if(strncasecmp(h->headers[i]+12,"keep-alive",10)==0)*/
-               if (strncasecmp(h->headers[i] + 12, "close", 5) == 0)
+          else if (strncasecmp("Connection:", h->headers[i], 11) == 0) {
+               val = h->headers[i] + 11;
+               for (;isspace(*val) && *val != '\0'; ++val);
+/*             if(strncasecmp(val,"keep-alive",10)==0)*/
+               if (strncasecmp(val, "close", 5) == 0)
                     req->keepalive = 0;
                /*else the default behaviour of keepalive ..... */
           }
@@ -466,7 +530,7 @@ static int parse_encaps_headers(ci_request_t * req)
      int size, i, request_status = 0;
      ci_encaps_entity_t *e = NULL;
      for (i = 0; (e = req->entities[i]) != NULL; i++) {
-          if (e->type > ICAP_RES_HDR)   //res_body,req_body or opt_body so the end of the headers.....
+          if (e->type > ICAP_RES_HDR)   //res_body,req_body or opt_body so the end of the headers.....process_encapsulated
                return EC_100;
 
           if (req->entities[i + 1] == NULL)
@@ -1422,18 +1486,20 @@ static int do_request(ci_request_t * req)
 
      auth_status = req->access_type;
      if ((auth_status = access_check_request(req)) == CI_ACCESS_DENY) {
+         req->keepalive = 0;
 	 if (req->auth_required) {
 	      ec_responce(req, EC_407); /*Responce with authentication required */
 	 }
 	 else {
              ec_responce(req, EC_403); /*Forbitten*/
 	 }
-          return CI_ERROR;      /*Or something that means authentication error */
+         return CI_ERROR;      /*Or something that means authentication error */
      }
 
      if (res == EC_100) {
           res = parse_encaps_headers(req);
           if (res != EC_100) {
+              req->keepalive = 0;
               ec_responce(req, EC_400);
               return CI_ERROR;
           }
@@ -1460,11 +1526,16 @@ static int do_request(ci_request_t * req)
               preview_status = do_request_preview(req);
           else {
               preview_status = do_fake_preview(req);
-              /*do_fake_preview return CI_OK or CI_ERROR.
-                We did not send any "100 Continue" response, but wee need 
-                to simulate that we sent...*/
-              if (preview_status == CI_OK) 
-                  req->return_code = EC_100;
+              /* do_fake_preview return CI_OK or CI_ERROR. */
+              if (preview_status == CI_OK) {
+                /* We did not send any "100 Continue" response, but we need
+                   to simulate that we sent in the case we are expecting
+                   body data */
+                  if (req->hasbody)
+                      req->return_code = EC_100;
+                  else
+                      req->return_code = EC_200;
+              }
           }
           
           if (preview_status == CI_ERROR) {
@@ -1535,6 +1606,10 @@ int process_request(ci_request_t * req)
     int res;
     ci_service_xdata_t *srv_xdata;
     res = do_request(req);
+
+    if (req->pstrblock_read_len) {
+        ci_debug_printf(5, "There are unparsed data od size %d: \"%s\"\n. Move to connection buffer\n", req->pstrblock_read_len, req->pstrblock_read);
+    }
    
     if (res<0 && req->request_header->bufused == 0) /*Did not read anything*/
 	return CI_NO_STATUS;
